// ignore_for_file: public_member_api_docs

import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';

part 'visualizer_page.g.dart';

///
/// ttt
///
@CopyWith()
class VisualizerPage extends StatelessWidget {
  ///
  ///
  ///
  const VisualizerPage({
    super.key,
    required this.ecgSequence,
    this.samplingRate = 250,
    this.strokeWidth = 1.5,
    this.isFiltered = true,
    this.isNormalized = true,
    this.measureTime = 60,
    this.color = Colors.black,
  });

  final int measureTime;
  final bool isFiltered;
  final double strokeWidth;
  final int samplingRate;
  final Iterable<double> ecgSequence;
  final bool isNormalized;

  final Color color;

  @override
  Widget build(BuildContext context) {
    final ecg = <FlSpot>[];
    double index;
    double minX;
    double maxX;
    double maxY;
    double minY;

    index = 0;
    minX = 0;
    maxX = samplingRate * measureTime.toDouble();
    maxY = 2;
    minY = -2;

    for (var y in ecgSequence) {
      if (isNormalized) {
        if (y > maxY) {
          y = isFiltered ? maxY : y;
        } else if (y < minY) {
          y = isFiltered ? minY : y;
        }
      }
      if (y != 0) {
        ecg.add(FlSpot(index, y));
      }
      index++;
    }

    return LineChart(
      LineChartData(
        minX: minX,
        maxX: maxX,
        minY: isFiltered ? minY : null,
        maxY: isFiltered ? maxY : null,
        backgroundColor: Colors.transparent,
        lineTouchData: LineTouchData(enabled: false),
        gridData: FlGridData(
          show: false,
        ),
        titlesData: FlTitlesData(
          show: false,
        ),
        borderData: FlBorderData(
          show: false,
        ),
        lineBarsData: [
          if (isFiltered)
            for (double i = 0; i <= maxX; i += samplingRate / 5)
              LineChartBarData(
                spots: [
                  FlSpot(i, minY),
                  FlSpot(i, maxY),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.3,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          if (isFiltered)
            for (double i = 0; i <= maxX; i += samplingRate / 25)
              LineChartBarData(
                spots: [
                  FlSpot(i, minY),
                  FlSpot(i, maxY),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.1,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          if (isFiltered)
            for (double i = minY; i <= maxY; i += 1)
              LineChartBarData(
                spots: [
                  FlSpot(minX, i),
                  FlSpot(maxX, i),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.3,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          if (isFiltered)
            for (double i = minY; i <= maxY; i += 1 / 5)
              LineChartBarData(
                spots: [
                  FlSpot(minX, i),
                  FlSpot(maxX, i),
                ],
                isCurved: false,
                color: Colors.blueGrey,
                barWidth: 0.1,
                dotData: FlDotData(
                  show: false,
                ),
              ),
          LineChartBarData(
            spots: ecg,
            color: color,
            isCurved: false,
            barWidth: strokeWidth,
            dotData: FlDotData(
              show: false,
            ),
          ),
        ],
      ),
      chartRendererKey: UniqueKey(),
      swapAnimationDuration: Duration.zero, // Optional
    );
  }
}
